//+------------------------------------------------------------------+
//|                                                 SpecialChart.mqh |
//|                        Copyright 2012, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"

#include <Canvas\Canvas.mqh>
#include <Arrays\ArrayDouble.mqh>
//+------------------------------------------------------------------+
//|             |
//+------------------------------------------------------------------+
class CSpecialChart: public CCanvas
  {
private:
   color             m_bkgroundcolor;  //  
   color             m_framecolor;     //  
   color             m_linecolor;      //  
   int               m_framewidth;     //    
   int               m_linewidth;      //    
   int               m_lines;          //    
   CArrayDouble      m_seria[];        //     
   bool              m_profitseria[];  //    
   int               m_lastseria_index;//     
   color             m_profit;         //   
   color             m_loss;           //   
public:
   //--- /
                     CSpecialChart():m_lastseria_index(0),m_profit(clrGreen),m_loss(clrRed){};
                    ~CSpecialChart() { CCanvas::Destroy(); }
   //---  ,   
   void              SetBackground(const color clr){ m_bkgroundcolor=clr;      };
   void              SetFrameColor(const color clr){ m_framecolor=clr;         };
   void              SetLineColor(const color clr) { m_linecolor=clr;          };
   //---    
   void              SetFrameWidth(const int w)    { m_framewidth=w;           };
   void              SetLineWidth(const int w)     { m_linewidth=w;            };
   //---     
   void              SetLines(const int l) { m_lines=l;ArrayResize(m_seria,l); ArrayResize(m_profitseria,l);};
   //---    
   void              Update();
   //---    
   void              AddSeria(const double  &array[],bool profit);
   //---   
   void              Draw(const int seria_index,color clr);
   //---                  color   uint
   uint              uCLR(const color clr) { return(XRGB((clr)&0x0FF,(clr)>>8,(clr)>>16));};
   //---     
   void              Line(int x1,int y1,int x2,int y2,uint col);
   //---  .      
   double            MaxValue(const int seria_index);
   double            MinValue(const int seria_index);
  };
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
void CSpecialChart::Update(void)
  {
//---  
   CCanvas::Erase(CSpecialChart::uCLR(m_bkgroundcolor));
//---  
   CCanvas::FillRectangle(m_framewidth,m_framewidth,
                          m_width-m_framewidth-1,
                          m_height-m_framewidth-1,
                          CSpecialChart::uCLR(m_framecolor));

//---     80%      
   for(int i=0;i<m_lines;i++)
     {
      color clr=m_loss;
      if(m_profitseria[i]) clr=m_profit;
      Draw(i,clr);
      //Print(__FUNCSIG__," - : ",i);
     }
//---  
   CCanvas::Update();
  }
//+------------------------------------------------------------------+
//|                      |
//+------------------------------------------------------------------+
void CSpecialChart::AddSeria(const double &array[],bool profit)
  {
//PrintFormat("     %d",m_lastseria_index);
   m_seria[m_lastseria_index].Resize(0);
   m_seria[m_lastseria_index].AddArray(array);
   m_profitseria[m_lastseria_index]=profit;
//---     (    )  
   m_lastseria_index++;
   if(m_lastseria_index>=m_lines) m_lastseria_index=0;
  }
//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
double CSpecialChart::MaxValue(const int seria_index)
  {
   double res=m_seria[seria_index].At(0);
   int total=m_seria[seria_index].Total();
//---   
   for(int i=1;i<total;i++)
     {
      if(m_seria[seria_index].At(i)>res) res=m_seria[seria_index].At(i);
     }
//--- 
   return res;
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
double CSpecialChart::MinValue(const int seria_index)
  {
   double res=m_seria[seria_index].At(0);;
   int total=m_seria[seria_index].Total();
//---     
   for(int i=1;i<total;i++)
     {
      if(m_seria[seria_index].At(i)<res) res=m_seria[seria_index].At(i);
     }
//--- 
   return res;
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
void CSpecialChart::Line(int x1,int y1,int x2,int y2,uint col)
  {
//---    Y   ,     y1  y2
   int y1_adj=m_height-y1;
   int y2_adj=m_height-y2;
//---         CSimpleCanvas
   CCanvas::Line(x1,y1_adj,x2,y2_adj,CSpecialChart::uCLR(col));
//---
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
void CSpecialChart::Draw(const int seria_index,color clr)
  {
//---       
   double min=MaxValue(seria_index);
   double max=MinValue(seria_index);
   double size=m_seria[seria_index].Total();
//---    
   double x_indent=m_width*0.1;
   double y_indent=m_height*0.1;
//---  
   double k_y=(max-min)/(m_height-2*m_framewidth-2*y_indent);
   double k_x=(size)/(m_width-2*m_framewidth-2*x_indent);
//--- 
   double start_x=x_indent;
   double start_y=m_height-y_indent;
//---         
   for(int i=1;i<size;i++)
     {
      //---    
      int x1=(int)((i-0)/k_x+start_x);  //     
      int y1=(int)(start_y-(m_seria[seria_index].At(i)-min)/k_y);    //  
      int x2=(int)((i-1-0)/k_x+start_x);//     
      int y2=(int)(start_y-(m_seria[seria_index].At(i-1)-min)/k_y);  //  
      //---       
      Line(x1,y1,x2,y2,clr);
     }
//---   
   CCanvas::Update();
  }
//+------------------------------------------------------------------+
